// Michael Chrien
// CS 378 Homework 2

/**************************************************************************

 Much of this physics and simulation code is from a nehe opengl tutorial.
 It has been heavily modified to suit my needs.
 For more information please see 
 **************************************************************************/

#include "stdafx.h"
#include <windows.h>               //header file for windows
#include <gl\gl.h>                 //header file for openGL
#include <gl\glu.h>                //header file for the openGL utility library
#include "glut.h"               //header file for GLUT
#include <conio.h>
#include <time.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <vector>

const GLfloat PI = 3.1459;

class Vector3D
{
public:
	float x;									// the x value of this Vector3D
	float y;									// the y value of this Vector3D
	float z;									// the z value of this Vector3D

	Vector3D()									// Constructor to set x = y = z = 0
	{
		x = 0;
		y = 0;
		z = 0;
	}

	Vector3D(float x, float y, float z)			// Constructor that initializes this Vector3D to the intended values of x, y and z
	{
		this->x = x;
		this->y = y;
		this->z = z;
	}

	Vector3D& operator= (Vector3D v)			// operator= sets values of v to this Vector3D. example: v1 = v2 means that values of v2 are set onto v1
	{
		x = v.x;
		y = v.y;
		z = v.z;
		return *this;
	}

	Vector3D operator+ (Vector3D v)				// operator+ is used to add two Vector3D's. operator+ returns a new Vector3D
	{
		return Vector3D(x + v.x, y + v.y, z + v.z);
	}

	Vector3D operator- (Vector3D v)				// operator- is used to take difference of two Vector3D's. operator- returns a new Vector3D
	{
		return Vector3D(x - v.x, y - v.y, z - v.z);
	}

	Vector3D operator* (float value)			// operator* is used to scale a Vector3D by a value. This value multiplies the Vector3D's x, y and z.
	{
		return Vector3D(x * value, y * value, z * value);
	}

	Vector3D operator/ (float value)			// operator/ is used to scale a Vector3D by a value. This value divides the Vector3D's x, y and z.
	{
		return Vector3D(x / value, y / value, z / value);
	}

	Vector3D& operator+= (Vector3D v)			// operator+= is used to add another Vector3D to this Vector3D.
	{
		x += v.x;
		y += v.y;
		z += v.z;
		return *this;
	}

	Vector3D& operator-= (Vector3D v)			// operator-= is used to subtract another Vector3D from this Vector3D.
	{
		x -= v.x;
		y -= v.y;
		z -= v.z;
		return *this;
	}

	Vector3D& operator*= (float value)			// operator*= is used to scale this Vector3D by a value.
	{
		x *= value;
		y *= value;
		z *= value;
		return *this;
	}

	Vector3D& operator/= (float value)			// operator/= is used to scale this Vector3D by a value.
	{
		x /= value;
		y /= value;
		z /= value;
		return *this;
	}


	Vector3D operator- ()						// operator- is used to set this Vector3D's x, y, and z to the negative of them.
	{
		return Vector3D(-x, -y, -z);
	}

	float length()								// length() returns the length of this Vector3D
	{
		return sqrtf(x*x + y*y + z*z);
	};			   		

	void unitize()								// unitize() normalizes this Vector3D that its direction remains the same but its length is 1.
	{
		float length = this->length();

		if (length == 0)
			return;

		x /= length;
		y /= length;
		z /= length;
	}

	Vector3D unit()								// unit() returns a new Vector3D. The returned value is a unitized version of this Vector3D.
	{
		float length = this->length();

		if (length == 0)
			return *this;
		
		return Vector3D(x / length, y / length, z / length);
	}

};

// class Mass			---> An object to represent a mass
class Mass
{
public:
	float m;									// The mass value
	Vector3D pos;								// Position in space
	Vector3D vel;								// Velocity
	Vector3D force;								// Force applied on this mass at an instance

	Mass(float m)								// Constructor
	{
		this->m = m;
	}

	/*
	  void applyForce(Vector3D force) method is used to add external force to the mass. 
	  At an instance in time, several sources of force might affect the mass. The vector sum 
	  of these forces make up the net force applied to the mass at the instance.
	*/
	void applyForce(Vector3D force)
	{
		this->force += force;					// The external force is added to the force of the mass
	}

	/*
	  void init() method sets the force values to zero
	*/
	void init()
	{
		force.x = 0;
		force.y = 0;
		force.z = 0;
	}

	/*
	  void simulate(float dt) method calculates the new velocity and new position of 
	  the mass according to change in time (dt). Here, a simulation method called
	  "The Euler Method" is used. The Euler Method is not always accurate, but it is 
	  simple. It is suitable for most of physical simulations that we know in common 
	  computer and video games.
	*/
	void simulate(float dt)
	{
		vel += (force / m) * dt;				// Change in velocity is added to the velocity.
												// The change is proportinal with the acceleration (force / m) and change in time

		pos += vel * dt;						// Change in position is added to the position.
												// Change in position is velocity times the change in time
	}

};

// class Simulation		---> A container object for simulating masses
class Simulation
{
public:
	int numOfMasses;								// number of masses in this container
	Mass** masses;									// masses are held by pointer to pointer. (Here Mass** represents a 1 dimensional array)
	
	Simulation(int numOfMasses, float m)			// Constructor creates some masses with mass values m
	{
		this->numOfMasses = numOfMasses;
		
		masses = new Mass*[numOfMasses];			// Create an array of pointers

		for (int a = 0; a < numOfMasses; ++a)		// We will step to every pointer in the array
			masses[a] = new Mass(m);				// Create a Mass as a pointer and put it in the array
	}

	virtual void release()							// delete the masses created
	{
		for (int a = 0; a < numOfMasses; ++a)		// we will delete all of them
		{
			delete(masses[a]);
			masses[a] = NULL;
		}
			
		delete(masses);
		masses = NULL;
	}

	Mass* getMass(int index)
	{
		if (index < 0 || index >= numOfMasses)		// if the index is not in the array
			return NULL;							// then return NULL

		return masses[index];						// get the mass at the index
	}

	virtual void init()								// this method will call the init() method of every mass
	{
		for (int a = 0; a < numOfMasses; ++a)		// We will init() every mass
			masses[a]->init();						// call init() method of the mass
	}

	virtual void solve()							// no implementation because no forces are wanted in this basic container
	{
													// in advanced containers, this method will be overrided and some forces will act on masses
	}

	virtual void simulate(float dt)					// Iterate the masses by the change in time
	{
		for (int a = 0; a < numOfMasses; ++a)		// We will iterate every mass
			masses[a]->simulate(dt);				// Iterate the mass and obtain new position and new velocity
	}

	virtual void operate(float dt)					// The complete procedure of simulation
	{
		init();										// Step 1: reset forces to zero
		solve();									// Step 2: apply forces
		simulate(dt);								// Step 3: iterate the masses by the change in time
	}

};

/*
  class ConstantVelocity is derived from class Simulation
  It creates 1 mass with mass value 1 kg and sets its velocity to (1.0f, 0.0f, 0.0f)
  so that the mass moves in the x direction with 1 m/s velocity.
*/
class ConstantVelocity : public Simulation
{
public:
	ConstantVelocity() : Simulation(1, 1.0f)				//Constructor firstly constructs its super class with 1 mass and 1 kg
	{
		masses[0]->pos = Vector3D(0.0f, 0.0f, 0.0f);		//a mass was created and we set its position to the origin
		masses[0]->vel = Vector3D(1.0f, 0.0f, 0.0f);		//we set the mass's velocity to (1.0f, 0.0f, 0.0f)
	}

};

/*
  class MotionUnderGravitation is derived from class Simulation
  It creates 1 mass with mass value 1 kg and sets its velocity to (10.0f, 15.0f, 0.0f) and its position to 
  (-10.0f, 0.0f, 0.0f). The purpose of this application is to apply a gravitational force to the mass and 
  observe the path it follows. The above velocity and position provides a fine projectile path with a 
  9.81 m/s/s downward gravitational acceleration. 9.81 m/s/s is a very close value to the gravitational 
  acceleration we experience on the earth.
*/
class MotionUnderGravitation : public Simulation
{
public:
	Vector3D gravitation;													//the gravitational acceleration

	MotionUnderGravitation(Vector3D gravitation) : Simulation(1, 1.0f)		//Constructor firstly constructs its super class with 1 mass and 1 kg
	{																		//Vector3D gravitation, is the gravitational acceleration
		this->gravitation = gravitation;									//set this class's gravitation
		masses[0]->pos = Vector3D(-10.0f, 0.0f, 0.0f);						//set the position of the mass
		masses[0]->vel = Vector3D(10.0f, 15.0f, 0.0f);						//set the velocity of the mass
	}

	virtual void solve()													//gravitational force will be applied therefore we need a "solve" method.
	{
		for (int a = 0; a < numOfMasses; ++a)								//we will apply force to all masses (actually we have 1 mass, but we can extend it in the future)
			masses[a]->applyForce(gravitation * masses[a]->m);				//gravitational force is as F = m * g. (mass times the gravitational acceleration)
	}
	
};

/*
  class MassConnectedWithSpring is derived from class Simulation
  It creates 1 mass with mass value 1 kg and binds the mass to an arbitrary constant point with a spring. 
  This point is refered as the connectionPos and the spring has a springConstant value to represent its 
  stiffness.
*/
class MassConnectedWithSpring : public Simulation
{
public:
	float springConstant;													//more the springConstant, stiffer the spring force
	Vector3D connectionPos;													//the arbitrary constant point that the mass is connected

	MassConnectedWithSpring(float springConstant) : Simulation(1, 1.0f)		//Constructor firstly constructs its super class with 1 mass and 1 kg
	{
		this->springConstant = springConstant;								//set the springConstant

		connectionPos = Vector3D(0.0f, -5.0f, 0.0f);						//set the connectionPos

		masses[0]->pos = connectionPos + Vector3D(10.0f, 0.0f, 0.0f);		//set the position of the mass 10 meters to the right side of the connectionPos
		masses[0]->vel = Vector3D(0.0f, 0.0f, 0.0f);						//set the velocity of the mass to zero
	}

	virtual void solve()													//the spring force will be applied
	{
		for (int a = 0; a < numOfMasses; ++a)								//we will apply force to all masses (actually we have 1 mass, but we can extend it in the future)
		{
			Vector3D springVector = masses[a]->pos - connectionPos;			//find a vector from the position of the mass to the connectionPos
			masses[a]->applyForce(-springVector * springConstant);			//apply the force according to the famous spring force formulation
		}
	}
	
};

/**************************************************************************

  File: Physics2.h  

  Prepared by Erkin Tunca for nehe.gamedev.net

**************************************************************************/

class Spring											//An object to represent a spring with inner friction binding two masses. The spring 
														//has a normal length (the length that the spring does not exert any force)
{
public:
	Mass* mass1;										//The first mass at one tip of the spring
	Mass* mass2;										//The second mass at the other tip of the spring

	float springConstant;								//A constant to represent the stiffness of the spring
	float springLength;									//The length that the spring does not exert any force
	float frictionConstant;								//A constant to be used for the inner friction of the spring

	Spring(Mass* mass1, Mass* mass2, 
		float springConstant, float springLength, float frictionConstant)		//Constructor
	{
		this->springConstant = springConstant;									//set the springConstant
		this->springLength = springLength;										//set the springLength
		this->frictionConstant = frictionConstant;								//set the frictionConstant

		this->mass1 = mass1;													//set mass1
		this->mass2 = mass2;													//set mass2
	}

	void solve()																	//solve() method: the method where forces can be applied
	{
		Vector3D springVector = mass1->pos - mass2->pos;							//vector between the two masses
		
		float r = springVector.length();											//distance between the two masses

		Vector3D force;																//force initially has a zero value
		
		if (r != 0)																	//to avoid a division by zero check if r is zero
			force += (springVector / r) * (r - springLength) * (-springConstant);	//the spring force is added to the force

		force += -(mass1->vel - mass2->vel) * frictionConstant;						//the friction force is added to the force
																					//with this addition we obtain the net force of the spring

		mass1->applyForce(force);													//force is applied to mass1
		mass2->applyForce(-force);													//the opposite of force is applied to mass2
	}

};

/*
  class RopeSimulation is derived from class Simulation (see Physics1.h). It simulates a rope with 
  point-like particles binded with springs. The springs have inner friction and normal length. One tip of 
  the rope is stabilized at a point in space called "Vector3D ropeConnectionPos". This point can be 
  moved externally by a method "void setRopeConnectionVel(Vector3D ropeConnectionVel)". RopeSimulation 
  creates air friction and a planer surface (or ground) with a normal in +y direction. RopeSimulation 
  implements the force applied by this surface. In the code, the surface is refered as "ground".
*/
class BloodSimulation : public Simulation				//An object to simulate a rope interacting with a planer surface and air
{
public:
	Spring** springs;									//Springs binding the masses (there shall be [numOfMasses - 1] of them)

	Vector3D gravitation;								//gravitational acceleration (gravity will be applied to all masses)

	Vector3D ropeConnectionPos;							//A point in space that is used to set the position of the 
														//first mass in the system (mass with index 0)
	
	Vector3D ropeConnectionVel;							//a variable to move the ropeConnectionPos (by this, we can swing the rope)

	float groundRepulsionConstant;						//a constant to represent how much the ground shall repel the masses
	
	float groundFrictionConstant;						//a constant of friction applied to masses by the ground
														//(used for the sliding of rope on the ground)
	
	float groundAbsorptionConstant;						//a constant of absorption friction applied to masses by the ground
														//(used for vertical collisions of the rope with the ground)
	
	float groundHeight;									//a value to represent the y position value of the ground
														//(the ground is a planer surface facing +y direction)

	float airFrictionConstant;							//a constant of air friction applied to masses

	bool amBall;
	BloodSimulation(										//a long long constructor with 11 parameters starts here
		int numOfMasses,								//1. the number of masses
		float m,										//2. weight of each mass
		float springConstant,							//3. how stiff the springs are
		float springLength,								//4. the length that a spring does not exert any force
		float springFrictionConstant,					//5. inner friction constant of spring
		Vector3D gravitation,							//6. gravitational acceleration
		float airFrictionConstant,						//7. air friction constant
		float groundRepulsionConstant,					//8. ground repulsion constant
		float groundFrictionConstant,					//9. ground friction constant
		float groundAbsorptionConstant,					//10. ground absorption constant
		float groundHeight,								//11. height of the ground (y position)
		float startHeight,
		float radius,
		bool ball = false
		) : Simulation(numOfMasses, m)					//The super class creates masses with weights m of each
	{
		this->gravitation = gravitation;
		this->amBall = ball;
		this->airFrictionConstant = airFrictionConstant;

		this->groundFrictionConstant = groundFrictionConstant;
		this->groundRepulsionConstant = groundRepulsionConstant;
		this->groundAbsorptionConstant = groundAbsorptionConstant;
		this->groundHeight = groundHeight;

		//double theta2, theta3;
		double r = 1.0;	
		Vector3D e,p;

		//int a;
		double theta2,theta3;
/*
		for (j=0;j<numOfMasses/2;j++) 
		{
			theta1 = j * 2 * PI / numOfMasses - PI / 2;
			theta2 = (j + 1) * 2 * PI / numOfMasses - PI / 2;

			for (i=0;i<numOfMasses;i++) 
			{
				theta3 = i * 2 * PI / numOfMasses;

				e.x = cos(theta2) * cos(theta3);
				e.y = sin(theta2);
				e.z = cos(theta2) * sin(theta3);
				p.x = 0.0f + 2.0 * e.x;
				p.y = 10.0f + 2.0 * e.y;
				p.z = 0.0f + 2.0 * e.z;

				//glNormal3f(e.x,e.y,e.z);
				//glTexCoord2f(i/(double)n,2*(j+1)/(double)n);
				//glVertex3f(p.x,p.y,p.z);

				masses[j*numOfMasses + i]->pos.x = p.x;		//Set x position of masses[a] with springLength distance to its neighbor
				masses[j*numOfMasses + i]->pos.y = p.y;						//Set y position as 0 so that it stand horizontal with respect to the ground
				masses[j*numOfMasses + i]->pos.z = p.z;						//Set z position as 0 so that it looks simple
			}
		}*/

		for (int a = 0; a < numOfMasses; ++a)			//To set the initial positions of masses loop with for(;;)
		{
			theta2 = (a + 1) *  PI / numOfMasses - PI / 2;
			theta3 = a * PI / numOfMasses;
			
			e.x = cos(theta2) * cos(theta3);
			e.y = sin(theta2);
			e.z = cos(theta2) * sin(theta3);
			p.x = 0.0f + radius * e.x;
			p.y = startHeight + radius * e.y;
			p.z = startHeight + radius * e.z;
			
			masses[a]->pos.x = p.x;		//Set x position of masses[a] with springLength distance to its neighbor
			masses[a]->pos.y = p.z;						//Set y position as 0 so that it stand horizontal with respect to the ground
			masses[a]->pos.z = 0.0f;						//Set z position as 0 so that it looks simple
		}

		if(ball)
			springs = new Spring*[4*numOfMasses];
		else
			springs = new Spring*[2*numOfMasses];			//create [numOfMasses - 1] pointers for springs
														//([numOfMasses - 1] springs are necessary for numOfMasses)
		int y = 0;
		for (a = 0; a < numOfMasses - 1; ++a)			//to create each spring, start a loop
		{
			Vector3D me = (masses[a]->pos - masses[a+1]->pos);
			double springLength2 = me.length();
			
			//Create the spring with index "a" by the mass with index "a" and another mass with index "a + 1".
			springs[y++] = new Spring(masses[a], masses[a + 1], 
				springConstant, springLength2, springFrictionConstant);
			
			int max = 1;
			if(ball)
				max = 3;
			for(int i = 0; i< max; ++i)
			{
				int ne = rand()%numOfMasses;
				me = (masses[ne]->pos - masses[a]->pos);
				springLength2 = me.length();
				if(!ball)
					springLength2*=2;
				//Create the spring with index "a" by the mass with index "a" and another mass with index "a + 1".
				springs[y++] = new Spring(masses[a], masses[ne], 
					springConstant, springLength2, springFrictionConstant);
			}
		}
		int max = 1;
		if(ball)
			max = 3;
		for(int i = 0; i< max; ++i)
		{
			int ne = rand()%numOfMasses;
			Vector3D me = (masses[ne]->pos - masses[numOfMasses-1]->pos);
			double springLength2 = me.length();
			if(!ball)
				springLength2*=2;
			//Create the spring with index "a" by the mass with index "a" and another mass with index "a + 1".
			springs[y++] = new Spring(masses[numOfMasses-1], masses[ne], 
				springConstant, springLength2, springFrictionConstant);
		}

		Vector3D me = (masses[numOfMasses-1]->pos - masses[0]->pos);
		double springLength2 = me.length();
		if(!ball)
				springLength2*=2;

		springs[y++] = new Spring(masses[numOfMasses-1], masses[0], 
			springConstant, springLength2, springFrictionConstant);
	}

	void release()										//release() is overriden because we have springs to delete
	{
		Simulation::release();							//Have the super class release itself

		for (int a = 0; a < 2*numOfMasses; ++a)		//to delete all springs, start a loop
		{
			delete(springs[a]);
			springs[a] = NULL;
		}
		
		delete(springs);
		springs = NULL;
	}

	void solve()										//solve() is overriden because we have forces to be applied
	{
		int a;
		int solver = 2;
		if(this->amBall)
			solver = 4;
		for (a = 0; a < solver*numOfMasses; ++a)		//apply force of all springs
		{
			springs[a]->solve();						//Spring with index "a" should apply its force
		}

		for (a = 0; a < numOfMasses; ++a)				//Start a loop to apply forces which are common for all masses
		{
			masses[a]->applyForce(gravitation * masses[a]->m);				//The gravitational force
			
			masses[a]->applyForce(-masses[a]->vel * airFrictionConstant);	//The air friction

			if (masses[a]->pos.y < groundHeight)		//Forces from the ground are applied if a mass collides with the ground
			{
				Vector3D v;								//A temporary Vector3D

				v = masses[a]->vel;						//get the velocity
				v.y = 0.0;								//omit the velocity component in y direction

				//The velocity in y direction is omited because we will apply a friction force to create 
				//a sliding effect. Sliding is parallel to the ground. Velocity in y direction will be used
				//in the absorption effect.
				masses[a]->applyForce(-v * groundFrictionConstant);		//ground friction force is applied

				v = masses[a]->vel;						//get the velocity
				v.x = 0;								//omit the x and z components of the velocity
				v.z = 0;								//we will use v in the absorption effect
				
				//above, we obtained a velocity which is vertical to the ground and it will be used in 
				//the absorption force

				if (v.y < 0)							//let's absorb energy only when a mass collides towards the ground
					masses[a]->applyForce(-v * groundAbsorptionConstant);		//the absorption force is applied
				
				//The ground shall repel a mass like a spring. 
				//By "Vector3D(0, groundRepulsionConstant, 0)" we create a vector in the plane normal direction 
				//with a magnitude of groundRepulsionConstant.
				//By (groundHeight - masses[a]->pos.y) we repel a mass as much as it crashes into the ground.
				Vector3D force = Vector3D(0, groundRepulsionConstant, 0) * 
					(groundHeight - masses[a]->pos.y);
				
			//	;;if(masses[a]->vel.y < -0.4)
			//	force.y = -200*masses[a]->vel.y;
				masses[a]->applyForce(force);			//The ground repulsion force is applied
			}
				
		}


	}

	void simulate(float dt)								//simulate(float dt) is overriden because we want to simulate 
														//the motion of the ropeConnectionPos
	{
		Simulation::simulate(dt);						//the super class shall simulate the masses

		ropeConnectionPos += ropeConnectionVel * dt;	//iterate the positon of ropeConnectionPos

		if (ropeConnectionPos.y < groundHeight)			//ropeConnectionPos shall not go under the ground
		{
			ropeConnectionPos.y = groundHeight;
			ropeConnectionVel.y = 0;
		}

	}

	void setRopeConnectionVel(Vector3D ropeConnectionVel)	//the method to set ropeConnectionVel
	{
		this->ropeConnectionVel = ropeConnectionVel;
	}

};









/**************************************************************************

 My rendering code begins here.

 **************************************************************************/

static void redraw(void);          //declarations
char frames [sizeof(long)*8+1];
char blood [sizeof(long)*8+1];
char balls [sizeof(long)*8+1];
long startTime;
long renders = 0;
int TICKSPEED = 2500/60;

int circleRes = 20;

int frame=0,curtime,timebase=0;
int cameraMode = 3;
int followDrop = 0;
GLfloat ballRotation = 0.0f;
GLfloat ballRotationFactor = 1.1f;

bool pause = false;

GLfloat zoomFactor = 0.0f;
GLfloat rotationFactor = 0.1f;

GLfloat ballVelocity = 0.0f;
GLfloat ballPosition = 10.0f;
long simulationTime = 0;

int windowWidth = 600;
int windowHeight = 600;
const double ELASTICITY = 0.80;

/* The dimensions of the viewing frustum */
GLfloat fleft   = -1.0;
GLfloat fright  =  1.0;
GLfloat fbottom = -1.0;
GLfloat ftop    =  1.0;
GLfloat zNear   = -2.0;
GLfloat zFar    = -7.0;

GLuint sphereTexture;
GLuint planeTexture;
GLuint ballTexture;

GLfloat angle = 0.0f;

struct Vector
{
	double x;
	double y;
	double z;
	double w;
};

GLfloat getPhysics(int time, bool updateVel)
{
	GLfloat accel = -0.00000098;

	if(updateVel)
	{
		ballVelocity += accel * time;
	}
	return ballVelocity * time;
}

std::vector <BloodSimulation*> newDroplets;
std::vector <BloodSimulation*> newBalls;

void setUpDroplets()
{

	BloodSimulation *drople;
	double initialHeight = 10.0f;

			drople = new BloodSimulation(
														circleRes,						// 80 Particles (Masses)
														0.2f,					// Each Particle Has A Weight Of 50 Grams
														0.05f,				// springConstant In The Rope
														0.2f,					// Normal Length Of Springs In The Rope
														0.5f,					// Spring Inner Friction Constant
														Vector3D(0, -3.8f,0),//-9.81f, 0), // Gravitational Acceleration
														0.002f,					// Air Friction Constant
														100.0f,					// Ground Repel Constant
														0.2f,					// Ground Slide Friction Constant
														5.0f,					// Ground Absoption Constant
														-1.35f,					// Height Of Ground
														initialHeight,
														3.0f);		

			newDroplets.push_back(drople);	



drople = new BloodSimulation(
														circleRes,						// 80 Particles (Masses)
														0.1f,					// Each Particle Has A Weight Of 50 Grams
														0.05f,				// springConstant In The Rope
														0.2f,					// Normal Length Of Springs In The Rope
														0.5f,					// Spring Inner Friction Constant
														Vector3D(0, -3.8f,0),//-9.81f, 0), // Gravitational Acceleration
														0.002f,					// Air Friction Constant
														1000.0f,					// Ground Repel Constant
														0.2f,					// Ground Slide Friction Constant
														50.0f,					// Ground Absoption Constant
														-1.35f,					// Height Of Ground
														initialHeight + 4.0f,
														1.3f);		

			newDroplets.push_back(drople);	

drople = new BloodSimulation(
														circleRes,						// 80 Particles (Masses)
														0.1f,					// Each Particle Has A Weight Of 50 Grams
														0.05f,				// springConstant In The Rope
														0.2f,					// Normal Length Of Springs In The Rope
														0.5f,					// Spring Inner Friction Constant
														Vector3D(0, -3.8f,0),//-9.81f, 0), // Gravitational Acceleration
														0.002f,					// Air Friction Constant
														10000.0f,					// Ground Repel Constant
														0.2f,					// Ground Slide Friction Constant
														50.0f,					// Ground Absoption Constant
														-1.35f,					// Height Of Ground
														initialHeight + 8.0f,
														0.9f);		

			newDroplets.push_back(drople);	

drople = new BloodSimulation(
														circleRes,						// 80 Particles (Masses)
														0.1f,					// Each Particle Has A Weight Of 50 Grams
														0.05f,				// springConstant In The Rope
														0.2f,					// Normal Length Of Springs In The Rope
														0.5f,					// Spring Inner Friction Constant
														Vector3D(0, -3.8f,0),//-9.81f, 0), // Gravitational Acceleration
														0.002f,					// Air Friction Constant
														10000.0f,					// Ground Repel Constant
														0.2f,					// Ground Slide Friction Constant
														50.0f,					// Ground Absoption Constant
														-1.35f,					// Height Of Ground
														initialHeight + 16.0f,
														1.0f);		

			newDroplets.push_back(drople);	
}




// load a 256x256 RGB .RAW file as a texture
GLuint LoadTextureRAW( const char * filename, int wrap )
{
    GLuint texture;
    int width, height;
    BYTE * data;
    FILE * file;

    // open texture data
    file = fopen( filename, "rb" );
    if ( file == NULL ) return 0;

    // allocate buffer
    width = 256;
    height = 256;
    data = (BYTE*)malloc( width * height * 3 );

    // read texture data
    fread( data, width * height * 3, 1, file );
    fclose( file );

    // allocate a texture name
    glGenTextures( 1, &texture );

    // select our current texture
    glBindTexture( GL_TEXTURE_2D, texture );

    // select modulate to mix texture with color for shading
    glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );

    // when texture area is small, bilinear filter the closest mipmap
    glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,
                     GL_LINEAR_MIPMAP_NEAREST );
    // when texture area is large, bilinear filter the first mipmap
    glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );

    // if wrap is true, the texture wraps over at the edges (repeat)
    //       ... false, the texture ends at the edges (clamp)
    glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S,
                     wrap ? GL_REPEAT : GL_CLAMP );
    glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T,
                     wrap ? GL_REPEAT : GL_CLAMP );

    // build our texture mipmaps
    gluBuild2DMipmaps( GL_TEXTURE_2D, 3, width, height,
                       GL_RGB, GL_UNSIGNED_BYTE, data );

    // free buffer
    free( data );

    return texture;
}


/*
   Create a sphere centered at c, with radius r, and precision n
   Draw a point for zero radius spheres
*/
void CreateSphere(double r,int n)
{

	// setup texture mapping
	glEnable( GL_TEXTURE_2D );
	glBindTexture( GL_TEXTURE_2D, sphereTexture );
	int i,j;
	double theta1,theta2,theta3;
	Vector e,p;

	if (r < 0)
	{
		r = -r;
	}
	if (n < 0)
	{
		n = -n;
	}
	if (n < 4 || r <= 0) 
	{
		glBegin(GL_POINTS);
		glVertex3f(0.0f,0.0f,0.0f);
		glEnd();
		return;
	}

	for (j=0;j<n/2;j++) 
	{
		theta1 = j * 2 * PI / n - PI / 2;
		theta2 = (j + 1) * 2 * PI / n - PI / 2;

		glBegin(GL_QUAD_STRIP);
		for (i=0;i<=n;i++) 
		{
			theta3 = i * 2 * PI / n;

			e.x = cos(theta2) * cos(theta3);
			e.y = sin(theta2);
			e.z = cos(theta2) * sin(theta3);
			p.x = 0.0f + r * e.x;
			p.y = 0.0f + r * e.y;
			p.z = 0.0f + r * e.z;

			glNormal3f(e.x,e.y,e.z);
			glTexCoord2f(i/(double)n,2*(j+1)/(double)n);
			glVertex3f(p.x,p.y,p.z);

			e.x = cos(theta1) * cos(theta3);
			e.y = sin(theta1);
			e.z = cos(theta1) * sin(theta3);
			p.x = 0.0f + r * e.x;
			p.y = 0.0f + r * e.y;
			p.z = 0.0f + r * e.z;

			glNormal3f(e.x,e.y,e.z);
			glTexCoord2f(i/(double)n,2*j/(double)n);
			glVertex3f(p.x,p.y,p.z);
		}
		glEnd();
	}
}

GLfloat getCenter(int num)
{
	if(num<(int)newDroplets.size())
	{
	Mass* mass1 = (newDroplets.at(num))->getMass(0);
	Vector3D* pos1 = &mass1->pos;

	Mass* mass2 = (newDroplets.at(num))->getMass((newDroplets.at(num))->numOfMasses/2);
	Vector3D* pos2 = &mass2->pos;

	return (pos1->y + pos2->y)/2;
	}
	return 1.0f;
	}
/*

void renderDrop(int num)
{
	// setup texture mapping
	glEnable( GL_TEXTURE_2D );
	glBindTexture( GL_TEXTURE_2D, sphereTexture );
	for (int i = 0; i < droplets[num]->numOfMasses; ++i)
	{
		double theta1 = 360.0 * ((double)i/(double)droplets[num]->numOfMasses) * 0.0174532925;//(2 * PI) / i;// (2* (i /20) ) *  PI / ropeSimulation->numOfMasses;
		double theta2 = 360.0 * ((double)(i+1)/(double)droplets[num]->numOfMasses) * 0.0174532925;//(2 * PI) / i;// (2* (i /20) ) *  PI / ropeSimulation->numOfMasses;

		double sa = sin(theta2);
		double as = cos(theta2);
		glColor3ub(255, 255, 0);
		glBegin(GL_QUAD_STRIP);
		for (int a = 0; a <= droplets[num]->numOfMasses; ++a)
		{
			Mass* mass1;
			if( a == droplets[num]->numOfMasses)
				mass1 = droplets[num]->getMass(0);
			else
				mass1 = droplets[num]->getMass(a);
			
			Vector3D* pos1 = &mass1->pos;

			Vector3D e;
			double theta3 = a * 2 * PI / droplets[num]->numOfMasses;

			e.x = cos(theta2) * cos(theta3);
			e.y = sin(theta2);
			e.z = cos(theta2) * sin(theta3);

			// Rotate to the right
			glNormal3f(e.x, e.y, e.z);
			glTexCoord2f(a/(double)droplets[num]->numOfMasses,2*(i+1)/(double)droplets[num]->numOfMasses);
			//glVertex3f(pos2->x *  cos(theta2), pos2->y , pos2->x  *  sin(theta2));
			glVertex3f(pos1->x *  cos(theta2), pos1->y , pos1->x  *  sin(theta2));
		
		
			e.x = cos(theta1) * cos(theta3);
			e.y = sin(theta1);
			e.z = cos(theta1) * sin(theta3);

			glNormal3f(e.x, e.y, e.z);
			glTexCoord2f(a/(double)droplets[num]->numOfMasses,2*i/(double)droplets[num]->numOfMasses);
			glVertex3f(pos1->x *  cos(theta1), pos1->y, pos1->x  *  sin(theta1));
		
		}
		glEnd();
	}
	glDisable( GL_TEXTURE_2D );
}
*/


void renderBallAdd(int num)
{		
	// setup texture mapping
	glEnable( GL_TEXTURE_2D );
	glBindTexture( GL_TEXTURE_2D, ballTexture );
	for (int i = 0; i < (newBalls.at(num))->numOfMasses; ++i)
	{
		double theta1 = 360.0 * ((double)i/(double)(newBalls.at(num))->numOfMasses) * 0.0174532925;//(2 * PI) / i;// (2* (i /20) ) *  PI / ropeSimulation->numOfMasses;
		double theta2 = 360.0 * ((double)(i+1)/(double)(newBalls.at(num))->numOfMasses) * 0.0174532925;//(2 * PI) / i;// (2* (i /20) ) *  PI / ropeSimulation->numOfMasses;

		double sa = sin(theta2);
		double as = cos(theta2);
		glColor3ub(255, 255, 0);
		glBegin(GL_QUAD_STRIP);
		for (int a = 0; a <= (newBalls.at(num))->numOfMasses; ++a)
		{
			Mass* mass1;
			if( a == (newBalls.at(num))->numOfMasses)
				mass1 = (newBalls.at(num))->getMass(0);
			else
				mass1 = (newBalls.at(num))->getMass(a);
			
			Vector3D* pos1 = &mass1->pos;

			Vector3D e;
			double theta3 = a * 2 * PI / (newBalls.at(num))->numOfMasses;

			e.x = cos(theta2) * cos(theta3);
			e.y = sin(theta2);
			e.z = cos(theta2) * sin(theta3);

			// Rotate to the right
			glNormal3f(e.x, e.y, e.z);
			glTexCoord2f(a/(double)(newBalls.at(num))->numOfMasses,2*(i+1)/(double)(newBalls.at(num))->numOfMasses);
			//glVertex3f(pos2->x *  cos(theta2), pos2->y , pos2->x  *  sin(theta2));
			glVertex3f(pos1->x *  cos(theta2), pos1->y , pos1->x  *  sin(theta2));
		
		
			e.x = cos(theta1) * cos(theta3);
			e.y = sin(theta1);
			e.z = cos(theta1) * sin(theta3);

			glNormal3f(e.x, e.y, e.z);
			glTexCoord2f(a/(double)(newBalls.at(num))->numOfMasses,2*i/(double)(newBalls.at(num))->numOfMasses);
			glVertex3f(pos1->x *  cos(theta1), pos1->y, pos1->x  *  sin(theta1));
		
		}
		glEnd();
	}
	glDisable( GL_TEXTURE_2D );
}

void renderDropAdd(int num)
{		
	// setup texture mapping
	glEnable( GL_TEXTURE_2D );
	glBindTexture( GL_TEXTURE_2D, sphereTexture );
	for (int i = 0; i < (newDroplets.at(num))->numOfMasses; ++i)
	{
		double theta1 = 360.0 * ((double)i/(double)(newDroplets.at(num))->numOfMasses) * 0.0174532925;//(2 * PI) / i;// (2* (i /20) ) *  PI / ropeSimulation->numOfMasses;
		double theta2 = 360.0 * ((double)(i+1)/(double)(newDroplets.at(num))->numOfMasses) * 0.0174532925;//(2 * PI) / i;// (2* (i /20) ) *  PI / ropeSimulation->numOfMasses;

		double sa = sin(theta2);
		double as = cos(theta2);
		glColor3ub(255, 255, 0);
		glBegin(GL_QUAD_STRIP);
		for (int a = 0; a <= (newDroplets.at(num))->numOfMasses; ++a)
		{
			Mass* mass1;
			if( a == (newDroplets.at(num))->numOfMasses)
				mass1 = (newDroplets.at(num))->getMass(0);
			else
				mass1 = (newDroplets.at(num))->getMass(a);
			
			Vector3D* pos1 = &mass1->pos;

			Vector3D e;
			double theta3 = a * 2 * PI / (newDroplets.at(num))->numOfMasses;

			e.x = cos(theta2) * cos(theta3);
			e.y = sin(theta2);
			e.z = cos(theta2) * sin(theta3);

			// Rotate to the right
			glNormal3f(e.x, e.y, e.z);
			glTexCoord2f(a/(double)(newDroplets.at(num))->numOfMasses,2*(i+1)/(double)(newDroplets.at(num))->numOfMasses);
			//glVertex3f(pos2->x *  cos(theta2), pos2->y , pos2->x  *  sin(theta2));
			glVertex3f(pos1->x *  cos(theta2), pos1->y , pos1->x  *  sin(theta2));
		
		
			e.x = cos(theta1) * cos(theta3);
			e.y = sin(theta1);
			e.z = cos(theta1) * sin(theta3);

			glNormal3f(e.x, e.y, e.z);
			glTexCoord2f(a/(double)(newDroplets.at(num))->numOfMasses,2*i/(double)(newDroplets.at(num))->numOfMasses);
			glVertex3f(pos1->x *  cos(theta1), pos1->y, pos1->x  *  sin(theta1));
		
		}
		glEnd();
	}
	glDisable( GL_TEXTURE_2D );
}



void renderSphere()
{
	float ambient[4]  = {0.9f, 0.5f, 0.5f, 1.0f} ;
	float diffuse[4]  = {0.9f, 0.5f, 0.5f, 1.0f} ;
	float specular[4]  = {0.9f, 0.5f, 0.5f, 1.0f} ;

	glMaterialfv( GL_FRONT, GL_AMBIENT,   ambient );
	glMaterialfv( GL_FRONT, GL_DIFFUSE,   diffuse );
	glMaterialfv( GL_FRONT, GL_SPECULAR,  specular );
	glMaterialf(  GL_FRONT, GL_SHININESS, 15.0f );
	
	// SPHERE!
	glPushMatrix();
	
	glTranslatef(0.0f, ballPosition + getPhysics(simulationTime, false), 0.0f);
	glRotatef(ballRotation, 1.0f, 0.0f, 0.0f); 

	glColor3f(1.0f, 0.0f, 1.0f);
	
	CreateSphere(0.5, 100);

	glPopMatrix();
}

void renderBlur()
{
	//printf("%f\n", ballVelocity);
	if( ballVelocity >= 0.0003f || ballVelocity <= -0.0003f )
	{
		
		float ambient[4]  = {0.2f, 0.2f, 0.2f, 0.1f} ;
		float diffuse[4]  = {0.2f, 0.2f, 0.2f, 1.0f} ;
		float specular[4]  = {0.2f, 0.2f, 0.2f, 0.1f} ;

		glMaterialfv( GL_FRONT, GL_AMBIENT,   ambient );
		glMaterialfv( GL_FRONT, GL_DIFFUSE,   diffuse );
		glMaterialfv( GL_FRONT, GL_SPECULAR,  specular );
		glMaterialf(  GL_FRONT, GL_SHININESS, 1.0f );
		
		// SPHERE!
		glPushMatrix();
		
		glTranslatef(0.0f, ballPosition + getPhysics(simulationTime-125, false), 0.0f);
		glRotatef(ballRotation, 1.0f, 0.0f, 0.0f); 

		CreateSphere(0.5, 100);

		glPopMatrix();

		ambient[0] = ambient[1] = ambient[2] = 0.1f;
		diffuse[0] = diffuse[1] = diffuse[2] = 0.1f;
		specular[0] = specular[1] = specular[2] = 0.1f;

		glMaterialfv( GL_FRONT, GL_AMBIENT,   ambient );
		glMaterialfv( GL_FRONT, GL_DIFFUSE,   diffuse );
		glMaterialfv( GL_FRONT, GL_SPECULAR,  specular );
		glMaterialf(  GL_FRONT, GL_SHININESS, 1.0f );
		
		// SPHERE!
		glPushMatrix();
		
		glTranslatef(0.0f, ballPosition + getPhysics(simulationTime-250, false), 0.0f);
		glRotatef(ballRotation, 1.0f, 0.0f, 0.0f); 

		CreateSphere(0.5, 100);

		glPopMatrix();

		ambient[0] = ambient[1] = ambient[2] = 0.05f;
		diffuse[0] = diffuse[1] = diffuse[2] = 0.05f;
		specular[0] = specular[1] = specular[2] = 0.05f;

		glMaterialfv( GL_FRONT, GL_AMBIENT,   ambient );
		glMaterialfv( GL_FRONT, GL_DIFFUSE,   diffuse );
		glMaterialfv( GL_FRONT, GL_SPECULAR,  specular );
		glMaterialf(  GL_FRONT, GL_SHININESS, 1.0f );
		
		// SPHERE!
		glPushMatrix();
		
		glTranslatef(0.0f, ballPosition + getPhysics(simulationTime-500, false), 0.0f);
		glRotatef(ballRotation, 1.0f, 0.0f, 0.0f); 

		CreateSphere(0.5, 100);

		glPopMatrix();
	}
}


void renderPlane(int size, double p_x, double p_z)
{
	glColor3f(1.0f,1.0f,1.0f);

	float ambient[4]  = {0.1f, 0.1f, 0.1f, 1.0f} ;
	float diffuse[4]  = {1.0f, 1.0f, 1.0f, 1.0f} ;
	float specular[4]  = {1.0f, 1.0f, 1.0f, 1.0f} ;

	glMaterialfv( GL_FRONT, GL_AMBIENT,   ambient );
	glMaterialfv( GL_FRONT, GL_DIFFUSE,   diffuse );
	glMaterialfv( GL_FRONT, GL_SPECULAR,  specular );
	glMaterialf(  GL_FRONT, GL_SHININESS, 128.0f );


	// setup texture mapping
	glEnable( GL_TEXTURE_2D );
	glBindTexture( GL_TEXTURE_2D, planeTexture );

	for(int x = 0; x<size; ++x)
	{
		for(int y = 0; y<size; ++y)
		{
			glPushMatrix();
			glTranslatef(0.0f, -1.5f, 0.0f);
			glBegin( GL_QUADS );
			
			glTexCoord2d(0.0,0.0); 
			glNormal3f(0.0f, 1.0f, 0.0f);
			glVertex3d(p_x + -1.0f + x * 2, 0.0f, p_z + -1.0f + y* 2);

			glTexCoord2d(1.0,0.0); 
			glNormal3f(0.0f, 1.0f, 0.0f);
			glVertex3d(p_x +  1.0f + x* 2, 0.0f, p_z + -1.0f + y* 2);

			glTexCoord2d(1.0,1.0); 
			glNormal3f(0.0f, 1.0f, 0.0f);
			glVertex3d(p_x +  1.0f + x* 2, 0.0f, p_z + 1.0f + y* 2);

			glTexCoord2d(0.0,1.0); 
			glNormal3f(0.0f, 1.0f, 0.0f);
			glVertex3d(p_x + -1.0f + x* 2, 0.0f, p_z + 1.0f + y* 2);
			
			glEnd();
			
			glPopMatrix();
		}
	}
}


void renderShadow()
{
	float ambient[4]  = {0.0f, 0.0f, 0.0f, 1.0f} ;
	float diffuse[4]  = {0.0f, 0.0f, 0.0f, 1.0f} ;
	float specular[4]  = {0.0f, 0.0f, 0.0f, 1.0f} ;

	glMaterialfv( GL_FRONT, GL_AMBIENT,   ambient );
	glMaterialfv( GL_FRONT, GL_DIFFUSE,   diffuse );
	glMaterialfv( GL_FRONT, GL_SPECULAR,  specular );
	glMaterialf(  GL_FRONT, GL_SHININESS, 0.0f );	

	glRotatef(90.0f, 1.0f, 0.0f, 0.0f);
	glPushMatrix();
	glColor3f(0.0f,0.0f,0.0f);
	glutSolidCone( 1.8f, 0.0f, 100, 100);
	glPopMatrix();
}


void enableLighting()
{
	float ambient0[4]  = {0.3f, 0.3f, 0.3f, 1.0f} ;
	float diffuse0[4]  = {0.8f, 0.8f, 0.8f, 1.0f} ;
	float specular0[4]  = {0.8f, 0.8f, 0.8f, 1.0f} ;
	//float position0[4]  = {0.0f, getCenter(followDrop) + 15.0f, 0.0f, 1.0f} ;
	float position0[4]  = {0.0f, 30.0f, 0.0f, 1.0f} ;
	float direction0[4]  = {0.0f, -1.0f, 0.0f, 1.0f} ;

	glLightfv( GL_LIGHT0, GL_AMBIENT, ambient0 );
	glLightfv( GL_LIGHT0, GL_DIFFUSE,  diffuse0 );
	glLightfv( GL_LIGHT0, GL_SPECULAR, specular0 );
	glLightfv( GL_LIGHT0, GL_POSITION, position0 );

	glLightfv( GL_LIGHT0, GL_SPOT_DIRECTION, direction0 );
	
	GLfloat angle = 30.0f;
	glLightf( GL_LIGHT0, GL_SPOT_CUTOFF, angle );
	GLfloat exp = 128.0f;
	glLightf( GL_LIGHT0, GL_SPOT_EXPONENT, exp );

	glLightf( GL_LIGHT0, GL_CONSTANT_ATTENUATION, 1.0f );
	glEnable( GL_LIGHT0 );



	float ambient1[4]  = {0.1f, 0.1f, 0.1f, 0.0f} ;
	float diffuse1[4]  = {0.2f, 0.2f, 0.2f, 1.0f} ;
	float specular1[4]  = {0.2f, 0.2f, 0.2f, 1.0f} ;
	float position1[4]  = {0.0f, 40.0f, 0.0f, 1.0f} ;

	glLightfv( GL_LIGHT1, GL_AMBIENT, ambient1 );
	glLightfv( GL_LIGHT1, GL_DIFFUSE,  diffuse1 );
	glLightfv( GL_LIGHT1, GL_SPECULAR, specular1 );
	glLightfv( GL_LIGHT1, GL_POSITION, position1 );

	glEnable( GL_LIGHT1 );
/*

	float ambient2[4]  = {0.0f, 0.0f, 0.0f, 0.0f} ;
	float diffuse2[4]  = {1.0f, 1.0f, 1.0f, 1.0f} ;
	float specular2[4]  = {1.0f, 1.0f, 1.0f, 1.0f} ;
	float position2[4]  = {10.0f, 1.0f, 10.0f, 1.0f} ;

	glLightfv( GL_LIGHT2, GL_AMBIENT, ambient2 );
	glLightfv( GL_LIGHT2, GL_DIFFUSE,  diffuse2 );
	glLightfv( GL_LIGHT2, GL_SPECULAR, specular2 );
	glLightfv( GL_LIGHT2, GL_POSITION, position2 );

	glEnable( GL_LIGHT2 );

	float ambient3[4]  = {0.0f, 0.0f, 0.0f, 0.0f} ;
	float diffuse3[4]  = {0.0f, 0.0f, 1.0f, 1.0f} ;
	float specular3[4]  = {0.0f, 0.0f, 1.0f, 1.0f} ;
	float position3[4]  = {-10.0f, 1.0f, -10.0f, 1.0f} ;

	glLightfv( GL_LIGHT3, GL_AMBIENT, ambient3 );
	glLightfv( GL_LIGHT3, GL_DIFFUSE,  diffuse3 );
	glLightfv( GL_LIGHT3, GL_SPECULAR, specular3 );
	glLightfv( GL_LIGHT3, GL_POSITION, position3 );

	glEnable( GL_LIGHT3 );
*/
	glEnable( GL_LIGHTING );
}

void disableLighting()
{
	glDisable( GL_LIGHT0 );
	glDisable( GL_LIGHT1 );
	glDisable( GL_LIGHT2 );
	glDisable( GL_LIGHT3 );
	glDisable( GL_LIGHTING );
}

void setOrthographicProjection() {

	// switch to projection mode
	glMatrixMode(GL_PROJECTION);
	// save previous matrix which contains the 
	//settings for the perspective projection
	glPushMatrix();
	// reset matrix
	glLoadIdentity();
	// set a 2D orthographic projection
	gluOrtho2D(0, windowWidth, 0, windowHeight);
	// invert the y axis, down is positive
	glScalef(1, -1, 1);
	// mover the origin from the bottom left corner
	// to the upper left corner
	glTranslatef(0, -windowHeight, 0);
	glMatrixMode(GL_MODELVIEW);
}

void resetPerspectiveProjection() {
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
}

void renderSpacedBitmapString(
			float x, 
			float y,
			int spacing, 
			void *font,
			char *string) {
  char *c;
  int x1=x;
  for (c=string; *c != '\0'; c++) {
	glRasterPos2f(x1,y);
    	glutBitmapCharacter(font, *c);
	x1 = x1 + glutBitmapWidth(font,*c) + spacing;
  }
}

/*
 * The main drawing routine.  Based on the current display mode, other
 * helper functions may be called.
 */
void myDisplay (void) {

	// Calculate the fps.


	// The camera follows the ball.
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective( 45.0f, (double)windowWidth/windowHeight, 1, 100 );
	glViewport( 0, 0, windowWidth, windowHeight );

	switch(cameraMode)
	{
		case 0:
			gluLookAt(0.0f, 1.0f, 7.5f + zoomFactor, 0.0f, getCenter(followDrop), 0.0f, 0.0f, 1.0f, 0.0f);
			glRotatef(angle, 0.0f, 1.0f, 0.0f); 
			break;
		case 1:
			gluLookAt(0.0f, getCenter(followDrop), 7.5f + zoomFactor, 0.0f, getCenter(followDrop), 0.0f, 0.0f, 1.0f, 0.0f);
			glRotatef(angle, 0.0f, 1.0f, 0.0f); 
			break;
		case 2:
			gluLookAt(0.0f, getCenter(followDrop) + 7.5f + zoomFactor, 0.0f, 0.0f, getCenter(followDrop), 0.0f, 0.0f, 0.0f, 1.0f);
			glRotatef(angle, 0.0f, 1.0f, 0.0f); 
			break;
		case 3:
			gluLookAt(12.5f + zoomFactor, getCenter(followDrop) + 12.5f + zoomFactor, 12.5f + zoomFactor, 0.0f, getCenter(followDrop), 0.0f, 0.0f, 1.0f, 0.0f);
			glRotatef(angle, 0.0f, 1.0f, 0.0f);
			break;
	}


/* Clear the pixels (aka colors) and the z-buffer */
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glMatrixMode(GL_MODELVIEW);	/* All matrix operations are for the model */
	glLoadIdentity();
	
	

	
	enableLighting();

	glEnable(GL_DEPTH_TEST);	/* Use the Z - buffer for visibility */

	renderPlane(24, -20, -20);

	//for(int bl=0; bl<NUM_DROPLETS; ++bl)
	//	renderDrop(bl);

	for(int bl=0; bl < (int)newDroplets.size(); ++bl)
		renderDropAdd(bl);

	for(int bl=0; bl < (int)newBalls.size(); ++bl)
		renderBallAdd(bl);
	//renderSphere();
	glLoadIdentity();


	if(newBalls.size()>0 || newDroplets.size()>0)
	{
			glTranslatef(0.0f, -1.49f, 0.0f);
	glScalef( (0.05)*(getCenter(followDrop) - (-1.5f)) + 1.0, 0.0f, (0.05)*(getCenter(followDrop) - (-1.5f)) + 1.0);
		renderShadow();
	}
	glColor4f(0.1f, 0.1f, 0.1f, 0.1f);   //defines the alpha value = 0.25f
   

	glEnable (GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_SRC_ALPHA);  

	glLoadIdentity();

	//renderBlur();

	itoa(circleRes, balls, 10);
	itoa((int)newDroplets.size(), blood, 10);

	glDisable(GL_BLEND);	

	disableLighting();
	setOrthographicProjection();
	glPushMatrix();
	glLoadIdentity();
	glColor3f(1.0f,1.0f,1.0f);
	renderSpacedBitmapString(5,15, 0, GLUT_BITMAP_HELVETICA_12, "Michael Chrien");
	renderSpacedBitmapString(5,30, 0, GLUT_BITMAP_HELVETICA_10, "Frames Per Second:");
	renderSpacedBitmapString(100,30, 0, GLUT_BITMAP_HELVETICA_10, frames);

	renderSpacedBitmapString(5,40, 0, GLUT_BITMAP_HELVETICA_10, "Blood Drops:");
	renderSpacedBitmapString(100,40, 0, GLUT_BITMAP_HELVETICA_10, blood);

	renderSpacedBitmapString(5,50, 0, GLUT_BITMAP_HELVETICA_10, "LOD:");
	renderSpacedBitmapString(100,50, 0, GLUT_BITMAP_HELVETICA_10, balls);


	renderSpacedBitmapString(5,60, 0, GLUT_BITMAP_HELVETICA_10, "'O' Views a different droplet");
	renderSpacedBitmapString(5,70, 0, GLUT_BITMAP_HELVETICA_10, "'P' pauses drops");
	renderSpacedBitmapString(5,80, 0, GLUT_BITMAP_HELVETICA_10, "'W'/'E' alter rotation speed");
	renderSpacedBitmapString(5,90, 0, GLUT_BITMAP_HELVETICA_10, "'A'/'Z' zoom in and out");
	renderSpacedBitmapString(5,100, 0, GLUT_BITMAP_HELVETICA_10, "'R' reset simulation");
	renderSpacedBitmapString(5,110, 0, GLUT_BITMAP_HELVETICA_10, "'B' add a new random blood drop");
	renderSpacedBitmapString(5,120, 0, GLUT_BITMAP_HELVETICA_10, "'N' delete a blood drop");
	renderSpacedBitmapString(5,130, 0, GLUT_BITMAP_HELVETICA_10, "'C' add a new random ball");
	renderSpacedBitmapString(5,140, 0, GLUT_BITMAP_HELVETICA_10, "'V' delete the ball");
	renderSpacedBitmapString(5,150, 0, GLUT_BITMAP_HELVETICA_10, "'M' delete everything");
	renderSpacedBitmapString(5,160, 0, GLUT_BITMAP_HELVETICA_10, "'K' increase LOD");
	renderSpacedBitmapString(5,170, 0, GLUT_BITMAP_HELVETICA_10, "'L' decrease LOD");
	glPopMatrix();
	resetPerspectiveProjection();


	glFlush();	/* Flush all executed OpenGL ops finish */
	
	/*
	 * Since we are using double buffers, we need to call the swap
	 * function every time we are done drawing.
	 */

	glutSwapBuffers();

	
	renders++;
	curtime=glutGet(GLUT_ELAPSED_TIME);
	if (curtime - timebase > 1000) {
		itoa(renders*1000.0/(curtime-timebase), frames, 10);
		timebase = curtime;		
		renders = 0;
	}

	glutPostRedisplay();
}

/*
 * Performs specific initializations for this program (as opposed to
 * glut initialization.
 */
void myInit (int argc, char **argv) 
{

	/* Set up a black background */
	glClearColor(0.1, 0.1, 0.1, 0.0);
	glEnable( GL_TEXTURE_2D );
	//glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT); 
	//glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
	//Set up our viewport such that coordinates are specified.
	//glViewport(0, 0, windowWidth, windowHeight);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	gluPerspective( 45.0f, (double)windowWidth/windowHeight, 1, 100 );
	glViewport( 0, 0, windowWidth, windowHeight );


	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	planeTexture = LoadTextureRAW( "./textures/concrete.raw", TRUE );
	sphereTexture = LoadTextureRAW( "./textures/blood.raw", TRUE );
	ballTexture = LoadTextureRAW( "./textures/basketball.raw", TRUE );
	startTime = time (NULL);

}

double velocity = 0;
double lastVelocity = 0;
double startPos = -1.0;
double origPos = -1.0;
bool reset = false;
//Increases the circles' size!


void myResize(int x, int y)
{
	windowWidth = x;
	windowHeight = y;
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	gluPerspective( 45.0f, (double)windowWidth/windowHeight, 1, 100 );
	glViewport( 0, 0, windowWidth, windowHeight );


	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
}

void tick(int i)
{
	angle += rotationFactor;
	if (simulationTime == 0)
	{
		i = 0;
		simulationTime = 1;
	}

	if(!pause)
	{
		float dt = i / 1000.0f;										// Let's Convert Milliseconds To Seconds

		float maxPossible_dt = 0.002f;											// Maximum Possible dt Is 0.002 Seconds
																				// This Is Needed To Prevent Pass Over Of A Non-Precise dt Value

  		int numOfIterations = (int)(dt / maxPossible_dt) + 1;					// Calculate Number Of Iterations To Be Made At This Update Depending On maxPossible_dt And dt
		if (numOfIterations != 0)												// Avoid Division By Zero
			dt = dt / numOfIterations;											// dt Should Be Updated According To numOfIterations

		for (int a = 0; a < numOfIterations; ++a)								// We Need To Iterate Simulations "numOfIterations" Times
		{
			for(int bl=0; bl < (int)newDroplets.size(); ++bl)
				(newDroplets.at(bl))->operate(dt);
			for(int bl=0; bl < (int)newBalls.size(); ++bl)
				(newBalls.at(bl))->operate(dt);
			
		}
		
	/*	if(i%500 == 1)
		{
			while(newDroplets.size() > 0)
				newDroplets.pop_back();
		}*/

		glutPostRedisplay();
		glutTimerFunc( TICKSPEED, tick, ++i);
	}
	else
	{
		glutPostRedisplay();
		glutTimerFunc( TICKSPEED, tick, i);
	}


	
}
void oldBall()
{
	if(!newBalls.empty())
		newBalls.erase(newBalls.begin());
}

void oldBlood()
{
	if(newDroplets.size()>1)
		newDroplets.erase(newDroplets.begin()+1);
}

void newBlood()
{
	BloodSimulation *drople;
	if(newDroplets.size() == 0)
	{
		simulationTime = 0;
		drople = new BloodSimulation(
													circleRes,						// 80 Particles (Masses)
													0.1f,					// Each Particle Has A Weight Of 50 Grams
													0.05f,				// springConstant In The Rope
													0.2f,					// Normal Length Of Springs In The Rope
													0.5f,					// Spring Inner Friction Constant
													Vector3D(0, -3.8f,0),//-9.81f, 0), // Gravitational Acceleration
													0.002f,					// Air Friction Constant
													10000.0f,					// Ground Repel Constant
													0.2f,					// Ground Slide Friction Constant
													50.0f,					// Ground Absoption Constant
													-1.35f,					// Height Of Ground
													rand()%20 + 1.0f,
													3.0f);	
	}
	else
	{
		drople = new BloodSimulation(
													circleRes,						// 80 Particles (Masses)
													0.1f,					// Each Particle Has A Weight Of 50 Grams
													0.05f,				// springConstant In The Rope
													0.2f,					// Normal Length Of Springs In The Rope
													0.5f,					// Spring Inner Friction Constant
													Vector3D(0, -3.8f,0),//-9.81f, 0), // Gravitational Acceleration
													0.002f,					// Air Friction Constant
													10000.0f,					// Ground Repel Constant
													0.2f,					// Ground Slide Friction Constant
													50.0f,					// Ground Absoption Constant
													-1.35f,					// Height Of Ground
													rand()%20 + 1.0f,
													rand()%2 + rand() %10 * 0.1f);	
	}

	newDroplets.push_back(drople);	

}	

void addBall()
{
	BloodSimulation *drople;
	simulationTime = 0;
	drople = new BloodSimulation(
												circleRes,						// 80 Particles (Masses)
												0.03f,					// Each Particle Has A Weight Of 50 Grams
												0.5f,				// springConstant In The Rope
												0.2f,					// Normal Length Of Springs In The Rope
												0.9f,					// Spring Inner Friction Constant
												Vector3D(0, -3.8f,0),//-9.81f, 0), // Gravitational Acceleration
												0.002f,					// Air Friction Constant
												5.0f,					// Ground Repel Constant
												0.0f,					// Ground Slide Friction Constant
												0.1f,					// Ground Absoption Constant
												-1.35f,					// Height Of Ground
												rand()%20 + 1.0f,
												3.5f, 
												true);	

	newBalls.push_back(drople);	

}	
/* Handle user input */
void myKeyHandler(unsigned char ch, int x, int y) {

	switch(ch) {
		case 'C':
		case 'c':
			oldBall();
			addBall();
			break;
		case 'M':
		case 'm':
			while(!newBalls.empty())
				newBalls.erase(newBalls.begin());
			while(!newDroplets.empty())
				newDroplets.erase(newDroplets.begin());

		case 'K':
		case 'k':
			++circleRes;
			break;
		case 'L':
		case 'l':
			if(circleRes>5)
				--circleRes;
			break;

		case 'V':
		case 'v':
			oldBall();
			break;
		case 'O':
		case 'o':
			++followDrop;
			if(followDrop == newDroplets.size())
				followDrop = 0;
			break;

		case 'b':
		case 'B':
			newBlood();
			break;


		case 'a':
		case 'A':
			zoomFactor -= 0.1f;
			break;
		case 'z':
		case 'Z':
			zoomFactor += 0.1f;
			break;

		case 'N':
		case 'n':
			oldBlood();
			break;

		case 'W':
		case 'w':
			rotationFactor -= 0.1f;
			break;
		case 'E':
		case 'e':
			rotationFactor += 0.1f;
			break;

		case 'R':
		case 'r':
			while(newDroplets.size() > 0)
				newDroplets.pop_back();
			while(newBalls.size() > 0)
				newBalls.pop_back();
			setUpDroplets();
				
			ballRotation = 0.0f;
			ballRotationFactor = 1.1f;

			pause = false;

			zoomFactor = 0.0f;
			rotationFactor = 0.1f;

			ballVelocity = 0.0f;
			ballPosition = 10.0f;
			simulationTime = 0;

			break;
		case 'P':
		case 'p':
			pause = !pause;
			break;

		default:
			/* Unrecognized key press, just return */
			return;

			break;
	}

	/*
	 * If control reaches here, the key press was recognized.  Refresh
	 * the screen, since most key presses change the display in some way.
	 */
	glutPostRedisplay();

	return;
}

int _tmain(int argc, _TCHAR* argv[])
{
	glutInit(&argc, argv);
setUpDroplets();
	/* Set initial window size and screen offset */
	glutInitWindowSize(windowWidth, windowHeight);
	glutInitWindowPosition(50, 50);

	/* Using: RGB (no alpha), double buffering, z-buffer */
	glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);

	glutCreateWindow("Michael Chrien's Blood Drop!");

	/* Set the function callbacks */
	glutDisplayFunc(myDisplay);
	glutReshapeFunc(myResize);
	glutKeyboardFunc(myKeyHandler);
	//glutMouseFunc(myMouseButton);
//	glutMotionFunc(myMouseMotion);
	//glutTimerFunc(1000, updateCircles, 1);
	glutTimerFunc( TICKSPEED, tick, 1);
	/* User specific initialization */
	myInit(argc, argv);
	
	glutMainLoop();	/* Go into the main glut control loop, will not return */

	/* Control flow will never reach here */
	return 0;
}
